@@ -13,7 +13,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -166,13 +166,24 @@ load_files (const char **files)
 		mode = EV_WINDOW_MODE_PRESENTATION;
 
 	for (i = 0; files[i]; i++) {
+		const gchar *filename;
 		gchar       *uri;
 		gchar       *label;
 		GFile       *file;
 		EvLinkDest  *dest = NULL;
 		const gchar *app_uri;
 
-		file = g_file_new_for_commandline_arg (files[i]);
+		filename = files[i];
+		label = strchr (filename, '#');
+		if (label) {
+			*label = 0;
+			label++;
+			dest = ev_link_dest_new_page_label (label);
+		} else if (global_dest) {
+			dest = g_object_ref (global_dest);
+		}
+
+		file = g_file_new_for_commandline_arg (filename);
 		uri = g_file_get_uri (file);
 		g_object_unref (file);
 
@@ -182,14 +193,7 @@ load_files (const char **files)
 			continue;
 		}
 
-		label = strchr (uri, '#');
-		if (label) {
-			*label = 0;
-			label++;
-			dest = ev_link_dest_new_page_label (label);
-		} else if (global_dest) {
-			dest = g_object_ref (global_dest);
-		}
+
 
 		ev_application_open_uri_at_dest (EV_APP, uri, screen, dest,
 						 mode, ev_find_string,
